==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ronportal/xml/classes/xmlelementreport.php <==
<?php
/**
 * stanislav.elbl, 21.06.2017
 */

namespace RonPortal\XML\Classes;


use Exception;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\System\PreparedReportOutputObject;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use RonPortal\System\ReportServer;
use RonPortal\User;
use RonPortal\XML\XMLFormElement;


class XMLElementReport extends XMLFormElement
{
    /** @var ReportServer */
    //var $reportServer;

    /** @var string */
    public $variablesXml = '';

    /** @var Report */
    private $report;


    /**
     * @throws DBException
     */
    private function getReport($reportName)
    {
        if (!$this ->report  ||  $this ->report ->kod !== $reportName) {
            $this ->report = Report::loadReport($this ->xmlForm ->database, $reportName);
        }

        return $this ->report;
    }


    /**
     * @throws DBException
     */
    private function getGlobalVariablesXml($reportName)
    {
        // TODO: vyresit promenne pro panely ze sestav v aplikaci?
        if (!$this ->getReport($reportName) ->appReportName) {
            // u starych reportu posle jeste puvodni parametry (pro kompatibilitu)
            return
//                "    <reportparam name='RC' value='" . StringUtils::xmlTextConvert($this->pp ? $this->pp->rc : '') . "'/>\n" .
//                "    <reportparam name='osobniCislo' value='" . StringUtils::xmlTextConvert($this->pp ? $this->pp->osobniCislo : '') . "'/>\n" .
//                "    <reportparam name='datumNastupu' type='DATE' value='" . ($this->pp ? date('j.n.Y', $this->pp->datumNastupu) : '1.1.1970') . "'/>\n" . // TODO: jake datum?
                // u starych sestav se pouzivala promenna LoginName
                "    <reportparam name='LoginName' value='" . StringUtils::htmlTextConvert(User::getLoginName()) . "'/>\n" .
                "    <reportparam name='RealLoginName' value='" . StringUtils::htmlTextConvert(User::getRealLoginName()) . "'/>\n";
        }

        return '';
    }

    /**
     * @throws DBException
     */
    function getXML()
    {
        $reportName = StringUtils::fromUTF8($this ->getAttributeValue('reportname'));
        $reportName = $this ->nahradPromenne($reportName);

//        if (!$this ->report  ||  $this ->report ->kod !== $reportName) {
//            $this ->report = Report::loadReport($this ->xmlForm ->database, $reportName);
//        }

//        // TODO: vyresit promenne pro panely ze sestav v aplikaci?
//        $globalVariablesXml = '';
//
//        if (!$this ->report ->appReportName) {
//            // u starych reportu posle jeste puvodni parametry (pro kompatibilitu)
//            $globalVariablesXml .=
//                // u starych sestav se pouzivala promenna LoginName
//                "    <reportparam name='LoginName' value='" . StringUtils::htmlTextConvert(User::getLoginName()) . "'/>\n" .
//                "    <reportparam name='RealLoginName' value='" . StringUtils::htmlTextConvert(User::getRealLoginName()) . "'/>\n";
//        }

        return
            "  <preparereport kod='" . StringUtils::xmlTextConvert($reportName) ."' sessionId='" . ReportServer::getReportSessionId() . "'>\n" .
            $this ->getGlobalVariablesXml($reportName) .
            $this ->variablesXml .
            "  </preparereport>";
    }


    private function getReportHtml($reportKey)
    {
        $text = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('text')));

        $format = $this ->getAttributeValue('format');
        $formatArray = $format ? explode(';', $format) : $this ->report ->nazvyFormatu;

        $attrs = [
            'class' => 'text_link report',
            'target' => '_blank'
        ];

        // doplni atributy oznacene prefixem html:
        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        if ($formatArray  &&  count($formatArray) > 1) {
            // jsou zadane formaty, napr. "pdf;xls"  nebo ";xls" (prvni se pouzije defaultni)
            $resultStr = '';
            $textZobrazen = false;

            foreach ($formatArray as $format_item) {
                if ($format_item) {
                    $resultStr .= " <a $attrs_str href='?menuid=xmlform&amp;action=report&amp;format={$format_item}&amp;reportkey={$reportKey}'>$format_item</a> ";
                } else {
                    // defaultni format
                    $resultStr .= " <a $attrs_str href='?menuid=xmlform&amp;action=report&amp;reportkey={$reportKey}'>$text</a> ";
                    $textZobrazen = true; // text uz se dale nebude zobrazovat
                }
            }

            $result =
                '<span class="reportlinks">' .
                ( $textZobrazen ? '' : $text) .
                $resultStr .
                "</span>\n";
        } else {
            // format neni zadan, nebo je zadan jenom jeden
            $formatstr = $format ? '&amp;format=' . $format : '';
            $result =
                "<a $attrs_str href='?menuid=xmlform&amp;action=report{$formatstr}&amp;reportkey={$reportKey}'>$text</a>";
        }

        return $result;
    }


    /**
     * @throws DBException
     */
    public function getHTMLOutput()
    {
        // zpracuje podrizene tagy - ty doplni $variablesXml
        $this ->variablesXml = '';
        $this ->getElementsHTMLOutput();

//        try {
//            $reportServer = ReportServer::createInstance();
//
//            $xmlResponse = $reportServer->sendXmlRequest($this->getXML());
//            $a = XmlLoader::loadXmlToArray($xmlResponse);
//
//            //var_dump($a);
//            $reportKey = $a['#attr']['key'];
//            //$reportKey = trim($this ->reportServer ->sendRequest('PREPAREREPORT ' . ReportServer::getReportSessionId() . "\n", true)); // jednoradkova odpoved = true
//
//            $reportServer ->sendMessage("EXIT\n");
//
//            $reportServer ->close();
//
//            $result = $this ->getReportHtml($reportKey);
//        } catch (Exception $e) {
//            $this ->xmlForm ->addErrorMsg($e ->getMessage());
//            return '';
//        }

        $reportName = StringUtils::fromUTF8($this ->getAttributeValue('reportname'));
        $reportName = $this ->nahradPromenne($reportName);

        $reportKey = PreparedReportOutputObject::addPreparedReport($reportName,
            $this ->variablesXml . $this ->getGlobalVariablesXml($reportName));

        return $this ->getReportHtml($reportKey);
    }
}


class XMLFormElementReportVariable extends XMLFormElement
{
    /**
     * @throws DBException
     * @throws Exception
     */
    public function getHTMLOutput()
    {
        /** @var XMLElementReport $reportElement */
        $reportElement = $this ->findParentByName('report');

        if (!$reportElement) {
            throw new Exception('Chyba ve formulari v tagu reportvariable - nenalezen nadrazeny tag report!');
        }

        $name = StringUtils::fromUTF8($this ->getAttributeValue('name'));
        $value = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('value')));

        switch ($this ->getAttributeValue('format')) {
            case 'int' :
            case 'integer' :
                $reportElement ->variablesXml .=
                    "    <reportparam name='$name' type='INT' value='$value'/>\n";
                break;

            case 'timestamp' :
                $reportElement ->variablesXml .=
                    "    <reportparam name='$name' type='DATE' value='" . date('j.n.Y H:i:s', $value) . "'/>\n";
                break;

            case 'obdobi' :
                $obdobi = Obdobi::vratObdobi($this ->xmlForm ->database, $this ->nahradPromenne($this ->getAttributeValue('aplikace')) ?: 'ADS', (int)$value);
                $reportElement ->variablesXml .=
                    ReportOutputObject::getXMLParamsObdobi(0, $obdobi ->zacatek, $obdobi ->konec, $obdobi ->popis);
                break;

            case 'pp' :
                $ppomer = ($value != ''  &&  (int)$value)
                    ? PracovniPomer::nactiPP($this ->xmlForm ->database, (int)$value)
                    : null;
                $reportElement ->variablesXml .=
                    ReportOutputObject::getXMLParamsOsoba($ppomer);

                break;

            default :
                $reportElement ->variablesXml .=
                    "    <reportparam name='$name' value='" . StringUtils::xmlTextConvert($value) . "'/>\n";
        }

        /*
        if (!$reportElement ->reportServer) {
            throw new \Exception('Chyba ve funkci questform_reportvariable - neni vytvoreno spojeni na report server!');
        }

        $pomstr = ($this ->getAttributeValue('format') == 'timestamp')
            ? 'DATEVAR ' . $name . '=' . date('j.n.Y H:i:s', $value)
            : 'VAR ' . $name . '=' . $value;

        $reportElement ->reportServer ->sendMessage($pomstr);
        */
    }
}

/*
//DOC
PAGE report
<pre>
PRE
  <report reportname="KOD_REPORTU" text="Zobrazený text" format="pdf;xls" >
    <reportvariable name="PROMENNA1" value="Hodnota promenne s parametry" />
    <reportvariable name="PROMENNA2" value="hodnota (timestamp)"  format="timestamp" />
    <reportvariable format="obdobi" value="obdobi (timestamp)" aplikace="ADS" />
    <reportvariable format="pp" value="idpp (int)" />
  </report>
PREEND
</pre>
//END
*/

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2311_175.zip: ronportal/ronportal/xml/classes/xmlelementreport.php <==
<?php
/**
 * stanislav.elbl, 21.06.2017
 */

namespace RonPortal\XML\Classes;


use Exception;
use Ron\DB\DBException;
use Ron\StringUtils;
use RonPortal\Obdobi;
use RonPortal\PracovniPomer;
use RonPortal\System\PreparedReportOutputObject;
use RonPortal\System\Report;
use RonPortal\System\ReportOutputObject;
use RonPortal\System\ReportServer;
use RonPortal\User;
use RonPortal\XML\XMLFormElement;


class XMLElementReport extends XMLFormElement
{
    /** @var ReportServer */
    //var $reportServer;

    /** @var string */
    public $variablesXml = '';

    /** @var Report */
    private $report;


    /**
     * @throws DBException
     */
    private function getReport($reportName)
    {
        if (!$this ->report  ||  $this ->report ->kod !== $reportName) {
            $this ->report = Report::loadReport($this ->xmlForm ->database, $reportName);
        }

        return $this ->report;
    }


    /**
     * @throws DBException
     */
    private function getGlobalVariablesXml($reportName)
    {
        // TODO: vyresit promenne pro panely ze sestav v aplikaci?
        if (!$this ->getReport($reportName) ->appReportName) {
            // u starych reportu posle jeste puvodni parametry (pro kompatibilitu)
            return
//                "    <reportparam name='RC' value='" . StringUtils::xmlTextConvert($this->pp ? $this->pp->rc : '') . "'/>\n" .
//                "    <reportparam name='osobniCislo' value='" . StringUtils::xmlTextConvert($this->pp ? $this->pp->osobniCislo : '') . "'/>\n" .
//                "    <reportparam name='datumNastupu' type='DATE' value='" . ($this->pp ? date('j.n.Y', $this->pp->datumNastupu) : '1.1.1970') . "'/>\n" . // TODO: jake datum?
                // u starych sestav se pouzivala promenna LoginName
                "    <reportparam name='LoginName' value='" . StringUtils::htmlTextConvert(User::getLoginName()) . "'/>\n" .
                "    <reportparam name='RealLoginName' value='" . StringUtils::htmlTextConvert(User::getRealLoginName()) . "'/>\n";
        }

        return '';
    }

//    /**
//     * @throws DBException
//     */
//    function getXML()
//    {
//        $reportName = StringUtils::fromUTF8($this ->getAttributeValue('reportname'));
//        $reportName = $this ->nahradPromenne($reportName);
//
//        return
//            "  <preparereport kod='" . StringUtils::xmlTextConvert($reportName) ."' sessionId='" . ReportServer::getReportSessionId() . "'>\n" .
//            $this ->getGlobalVariablesXml($reportName) .
//            $this ->variablesXml .
//            "  </preparereport>";
//    }


    private function getReportHtml($reportKey)
    {
        $text = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('text')));

        $format = $this ->getAttributeValue('format');
        $formatArray = $format ? explode(';', $format) : $this ->report ->nazvyFormatu;

        $attrs = [
            'class' => 'text_link report',
            'target' => '_blank'
        ];

        // doplni atributy oznacene prefixem html:
        $attrs = $this->setHTMLAttributes($attrs);
        $attrs_str = $this->getHTMLAttributes($attrs);

        if ($formatArray  &&  count($formatArray) > 1) {
            // jsou zadane formaty, napr. "pdf;xls"  nebo ";xls" (prvni se pouzije defaultni)
            $resultStr = '';
            $textZobrazen = false;

            foreach ($formatArray as $format_item) {
                if ($format_item) {
                    $resultStr .= " <a $attrs_str href='?menuid=xmlform&amp;action=report&amp;format={$format_item}&amp;reportkey={$reportKey}'>$format_item</a> ";
                } else {
                    // defaultni format
                    $resultStr .= " <a $attrs_str href='?menuid=xmlform&amp;action=report&amp;reportkey={$reportKey}'>$text</a> ";
                    $textZobrazen = true; // text uz se dale nebude zobrazovat
                }
            }

            $result =
                '<span class="reportlinks">' .
                ( $textZobrazen ? '' : $text) .
                $resultStr .
                "</span>\n";
        } else {
            // format neni zadan, nebo je zadan jenom jeden
            $formatstr = $format ? '&amp;format=' . $format : '';
            $result =
                "<a $attrs_str href='?menuid=xmlform&amp;action=report{$formatstr}&amp;reportkey={$reportKey}'>$text</a>";
        }

        return $result;
    }


    /**
     * @throws DBException
     */
    public function getHTMLOutput()
    {
        // zpracuje podrizene tagy - ty doplni $variablesXml
        $this ->variablesXml = '';
        $this ->getElementsHTMLOutput();

//        try {
//            $reportServer = ReportServer::createInstance();
//
//            $xmlResponse = $reportServer->sendXmlRequest($this->getXML());
//            $a = XmlLoader::loadXmlToArray($xmlResponse);
//
//            //var_dump($a);
//            $reportKey = $a['#attr']['key'];
//            //$reportKey = trim($this ->reportServer ->sendRequest('PREPAREREPORT ' . ReportServer::getReportSessionId() . "\n", true)); // jednoradkova odpoved = true
//
//            $reportServer ->sendMessage("EXIT\n");
//
//            $reportServer ->close();
//
//            $result = $this ->getReportHtml($reportKey);
//        } catch (Exception $e) {
//            $this ->xmlForm ->addErrorMsg($e ->getMessage());
//            return '';
//        }

        $reportName = StringUtils::fromUTF8($this ->getAttributeValue('reportname'));
        $reportName = $this ->nahradPromenne($reportName);

        $reportKey = PreparedReportOutputObject::addPreparedReport($reportName,
            $this ->variablesXml . $this ->getGlobalVariablesXml($reportName));

        return $this ->getReportHtml($reportKey);
    }
}


class XMLFormElementReportVariable extends XMLFormElement
{
    /**
     * @throws DBException
     * @throws Exception
     */
    public function getHTMLOutput()
    {
        /** @var XMLElementReport $reportElement */
        $reportElement = $this ->findParentByName('report');

        if (!$reportElement) {
            throw new Exception('Chyba ve formulari v tagu reportvariable - nenalezen nadrazeny tag report!');
        }

        $name = StringUtils::fromUTF8($this ->getAttributeValue('name'));
        $value = $this ->nahradPromenne(StringUtils::fromUTF8($this ->getAttributeValue('value')));

        switch ($this ->getAttributeValue('format')) {
            case 'int' :
            case 'integer' :
                $reportElement ->variablesXml .=
                    "    <reportparam name='$name' type='INT' value='$value'/>\n";
                break;

            case 'timestamp' :
                $reportElement ->variablesXml .=
                    "    <reportparam name='$name' type='DATE' value='" . date('j.n.Y H:i:s', $value) . "'/>\n";
                break;

            case 'obdobi' :
                $obdobi = Obdobi::vratObdobi($this ->xmlForm ->database, $this ->nahradPromenne($this ->getAttributeValue('aplikace')) ?: 'ADS', (int)$value);
                $reportElement ->variablesXml .=
                    ReportOutputObject::getXMLParamsObdobi(0, $obdobi ->zacatek, $obdobi ->konec, $obdobi ->popis);
                break;

            case 'pp' :
                $ppomer = ($value != ''  &&  (int)$value)
                    ? PracovniPomer::nactiPP($this ->xmlForm ->database, (int)$value)
                    : null;
                $reportElement ->variablesXml .=
                    ReportOutputObject::getXMLParamsOsoba($ppomer);

                break;

            default :
                $reportElement ->variablesXml .=
                    "    <reportparam name='$name' value='" . StringUtils::xmlTextConvert($value) . "'/>\n";
        }

        /*
        if (!$reportElement ->reportServer) {
            throw new \Exception('Chyba ve funkci questform_reportvariable - neni vytvoreno spojeni na report server!');
        }

        $pomstr = ($this ->getAttributeValue('format') == 'timestamp')
            ? 'DATEVAR ' . $name . '=' . date('j.n.Y H:i:s', $value)
            : 'VAR ' . $name . '=' . $value;

        $reportElement ->reportServer ->sendMessage($pomstr);
        */
    }
}

/*
//DOC
PAGE report
<pre>
PRE
  <report reportname="KOD_REPORTU" text="Zobrazený text" format="pdf;xls" >
    <reportvariable name="PROMENNA1" value="Hodnota promenne s parametry" />
    <reportvariable name="PROMENNA2" value="hodnota (timestamp)"  format="timestamp" />
    <reportvariable format="obdobi" value="obdobi (timestamp)" aplikace="ADS" />
    <reportvariable format="pp" value="idpp (int)" />
  </report>
PREEND
</pre>
//END
*/
